@@ -37,4 +37,6 @@ echo ""
 ./abi-extract.pl -N ko
 ./abi-extract.pl -N ne-NP
 ./abi-extract.pl -N ur
-
+./abi-extract.pl -N wo-SN
+./abi-extract.pl -N ps
+./abi-extract.pl -N "be@latin.po"
