@@ -21,6 +21,10 @@
  * USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #if 0
 #include <goffice/goffice-config.h>
 #include <glib/gi18n-lib.h>
@@ -38,7 +42,11 @@
 #include <glib/gstdio.h>
 #include <libxml/encoding.h>
 
-#ifdef HAVE_GNOMEVFS
+#ifdef WITH_GSF_INPUT_HTTP
+#include <gsf/gsf-input-http.h>
+#endif
+
+#ifdef WITH_GNOMEVFS
 #define GOFFICE_WITH_GNOME
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -47,7 +55,7 @@
 #include <gsf-gnome/gsf-output-gnomevfs.h>
 #endif
 
-#ifdef HAVE_GNOMEUI
+#ifdef WITH_GNOMEUI
 #include <libgnome/gnome-url.h>
 #endif
 
@@ -55,6 +63,7 @@
 #include <windows.h>
 #include <shellapi.h>
 #include <io.h>
+#include <fcntl.h>
 #endif
 
 #include <string.h>
@@ -74,7 +83,7 @@ GsfInput *
 gsf_input_memory_new_from_file (FILE * input)
 {
 	GsfOutput *memory_output;
-	GsfInput  *memory_input;
+	GsfInput  *memory_input = NULL;
 
 	g_return_val_if_fail (input != NULL, NULL);
 
@@ -82,17 +91,23 @@ gsf_input_memory_new_from_file (FILE * input)
 	while (TRUE) {
 		guint8 buf[1024];
 		size_t nread;
+		gboolean res;
 
 		nread = fread (buf, 1, sizeof(buf), input);
-		gsf_output_write (memory_output, nread, buf);
-		if ((nread < sizeof(buf)) && (ferror (input) || feof (input)))
-		    break;
+		res = gsf_output_write (memory_output, nread, buf);
+
+		if (ferror (input) || !res) {
+			/* trouble reading from @input or trouble writing to @memory_output */
+			g_object_unref (G_OBJECT (memory_output));
+			return NULL;
+		}
+		else if ((nread < sizeof(buf)) && feof (input)) /* hit eof */
+			break;
 	}
 
-	gsf_output_close (memory_output);
-
-	memory_input = gsf_input_memory_new_clone (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (memory_output)),
-						   gsf_output_size (memory_output));
+	if (gsf_output_close (memory_output))
+		memory_input = gsf_input_memory_new_clone (gsf_output_memory_get_bytes (GSF_OUTPUT_MEMORY (memory_output)),
+							   gsf_output_size (memory_output));
 
 	g_object_unref (G_OBJECT (memory_output));
 
@@ -913,6 +928,12 @@ UT_go_shell_arg_to_uri (const char *arg)
 			return UT_go_url_simplify (arg);
 		}
 	}
+#elif defined(WITH_GSF_INPUT_HTTP)
+	{
+		if (g_ascii_strncasecmp (arg, "http://", strlen ("http://")) == 0) {
+			return UT_go_url_simplify (arg);
+		}
+	}
 #endif
 
 	/* Just assume it's a filename.  */
@@ -1077,6 +1098,9 @@ UT_go_file_open_impl (char const *uri, GError **err)
 	}
 
 	if (is_fd_uri (uri, &fd)) {
+#if defined G_OS_WIN32
+		setmode (fd, O_BINARY);
+#endif
 		int fd2 = dup (fd);
 		FILE *fil = fd2 != -1 ? fdopen (fd2, "rb") : NULL;
 		GsfInput *result;
@@ -1096,11 +1120,27 @@ UT_go_file_open_impl (char const *uri, GError **err)
 
 #ifdef GOFFICE_WITH_GNOME
 	return gsf_input_gnomevfs_new (uri, err);
-#else
+#elif defined(WITH_GSF_INPUT_HTTP)
+	{
+		GsfInput *http = gsf_input_http_new (uri, err);
+		
+		if (http != NULL) {
+			GsfInput *result;
+			gsf_off_t size;
+
+			/* guarantee that file descriptors will be seekable */
+			size = gsf_input_size (http);
+			result = gsf_input_memory_new_clone (gsf_input_read (http, size, NULL), size);
+
+			g_object_unref (G_OBJECT (http));
+
+			return result;
+		}
+	}
+#endif
 	g_set_error (err, gsf_input_error (), 0,
 		     "Invalid or non-supported URI");
 	return NULL;
-#endif
 }
 
 /**
@@ -1141,6 +1181,9 @@ UT_go_file_create_impl (char const *uri, GError **err)
 	}
 
 	if (is_fd_uri (uri, &fd)) {
+#if defined G_OS_WIN32
+		setmode (fd, O_BINARY);
+#endif
 		int fd2 = dup (fd);
 		FILE *fil = fd2 != -1 ? fdopen (fd2, "wb") : NULL;
 		GsfOutput *result = fil ? gsf_output_stdio_new_FILE (uri, fil, FALSE) : NULL;
@@ -1178,6 +1221,30 @@ UT_go_file_create (char const *uri, GError **err)
 	return NULL;
 }
 
+gboolean
+UT_go_file_remove (char const *uri, GError **err)
+{
+	char *filename;
+
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	filename = UT_go_filename_from_uri (uri);
+	if (filename) {
+		int result = remove (filename);
+		g_free (filename);
+		return (result == 0);
+	}
+
+
+#ifdef GOFFICE_WITH_GNOME
+	return (gnome_vfs_unlink (uri) == GNOME_VFS_OK);
+#else
+	g_set_error (err, gsf_output_error_id (), 0,
+		     "Invalid or non-supported URI");
+	return FALSE;
+#endif
+}
+
 /* ------------------------------------------------------------------------- */
 /* Adapted from gtkfilechooserdefault.c.  Unfortunately it is static there.  */
 
@@ -1560,7 +1627,7 @@ UT_go_url_encode (gchar const *text, int type)
 	return g_string_free (result, FALSE);
 }
 
-#ifndef HAVE_GNOMEUI
+#ifndef WITH_GNOMEUI
 static char *
 check_program (char const *prog)
 {
@@ -1584,7 +1651,7 @@ UT_go_url_show (gchar const *url)
 	return NULL;
 #else
 	GError *err = NULL;
-#ifdef HAVE_GNOMEUI
+#ifdef WITH_GNOMEUI
 	gnome_url_show (url, &err);
 	return err;
 #else
