@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2003 Martin Sevior (msevior@physics.unimelb.edu.au>
@@ -98,7 +100,8 @@ fl_FrameLayout::fl_FrameLayout(FL_DocLayout* pLayout,
 	  m_iYPage(0),
 	  m_iBoundingSpace(0),
 	  m_iFrameWrapMode(FL_FRAME_ABOVE_TEXT),
-	  m_bIsTightWrap(false)
+	  m_bIsTightWrap(false),
+	  m_iPrefPage(-1)
 {
 	UT_ASSERT(m_pDocSL->getContainerType() == FL_CONTAINER_DOCSECTION);
 }
@@ -187,7 +190,7 @@ void 	fl_FrameLayout::setContainerProperties(void)
 			{
 				UT_sint32 iImageWidth;
 				UT_sint32 iImageHeight;
-				UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(m_pGraphicImage)->getRaster_PNG();
+				const UT_ByteBuf * pBB = static_cast<FG_GraphicRaster *>(m_pGraphicImage)->getRaster_PNG();
 				UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 				iWidth = pG->tlu(iImageWidth);
 				iHeight = pG->tlu(iImageHeight);
@@ -220,6 +223,7 @@ void 	fl_FrameLayout::setContainerProperties(void)
 //
 		pFrame->setWrapping(true);
 	}
+	pFrame->setPreferedPageNo(m_iPrefPage);
 }
 
 UT_sint32 fl_FrameLayout::getBoundingSpace(void) const
@@ -372,25 +376,33 @@ bool fl_FrameLayout::doclistener_changeStrux(const PX_ChangeRecord_StruxChange *
 {
 	UT_ASSERT(pcrxc->getType()==PX_ChangeRecord::PXT_ChangeStrux);
 	fp_FrameContainer * pFrameC = static_cast<fp_FrameContainer *>(getFirstContainer());
-	UT_GenericVector<fl_BlockLayout *> vecBlocks;
-	pFrameC->getBlocksAroundFrame(vecBlocks);
+	UT_GenericVector<fl_ContainerLayout *> AllLayouts;
+	AllLayouts.clear();
+	fp_Page * pPage = NULL;
 	UT_uint32 i = 0;
-	for(i=0; i< vecBlocks.getItemCount();i++)
+	if(pFrameC)
 	{
-	  fl_BlockLayout * pBL = vecBlocks.getNthItem(i);
-	  pBL->collapse();
-	  xxx_UT_DEBUGMSG(("Collapse block %x \n",pBL));
+	    pPage = pFrameC->getPage();
+	    UT_return_val_if_fail(pPage, false);
+	    pPage->getAllLayouts(AllLayouts);
+	    for(i=0; i< AllLayouts.getItemCount();i++)
+	    {
+	         fl_ContainerLayout * pCL = AllLayouts.getNthItem(i);
+		 pCL->collapse();
+	    }
 	}
 	setAttrPropIndex(pcrxc->getIndexAP());
 	collapse();
 	lookupProperties();
 	format();
-	for(i=0; i< vecBlocks.getItemCount();i++)
+	for(i=0; i< AllLayouts.getItemCount();i++)
 	{
-	  fl_BlockLayout * pBL = vecBlocks.getNthItem(i);
-	  pBL->format();
-	  xxx_UT_DEBUGMSG(("Format block %x \n",pBL));
+	    fl_ContainerLayout * pCL = AllLayouts.getNthItem(i);
+	    pCL->format();
+	    xxx_UT_DEBUGMSG(("Format block %x \n",pBL));
+	    pCL->markAllRunsDirty();
 	}
+	getDocSectionLayout()->markAllRunsDirty();
 	return true;
 }
 
@@ -805,7 +817,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 	const gchar * pszBoundingSpace = NULL;
 	const gchar * pszTightWrapped = NULL;
-
+	const gchar * pszPrefPage = NULL;
 // Frame Type
 
 	if(!pSectionAP || !pSectionAP->getProperty("frame-type",pszFrameType))
@@ -829,8 +841,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 // Position-to
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("position-to",pszPositionTo))
+	if((!pSectionAP || !pSectionAP->getProperty("position-to",pszPositionTo)))
 	{
 		m_iFramePositionTo = FL_FRAME_POSITIONED_TO_BLOCK;
 	}
@@ -855,12 +866,8 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 
 // wrap-mode
-
-	if(pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER))
-	{
-		m_iFrameWrapMode = FL_FRAME_WRAPPED_TO_RIGHT;
-	}
-	else if(!pSectionAP || !pSectionAP->getProperty("wrap-mode",pszWrapMode))
+	
+	if(!pSectionAP || !pSectionAP->getProperty("wrap-mode",pszWrapMode))
 	{
 		m_iFrameWrapMode = FL_FRAME_ABOVE_TEXT;
 	}
@@ -897,8 +904,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	//
 	// Wrap type
 	//
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("tight-wrap",pszTightWrapped))
+	if(!pSectionAP || !pSectionAP->getProperty("tight-wrap",pszTightWrapped))
 	{
 		m_bIsTightWrap = false;
 	}
@@ -913,8 +919,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 // Xpos
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("xpos",pszXpos))
+	if(!pSectionAP || !pSectionAP->getProperty("xpos",pszXpos))
 	{
 		m_iXpos = 0;
 	}
@@ -925,8 +930,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	UT_DEBUGMSG(("xpos for frame is %s \n",pszXpos));
 // Ypos
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("ypos",pszYpos))
+	if(!pSectionAP || !pSectionAP->getProperty("ypos",pszYpos))
 	{
 		m_iYpos = 0;
 	}
@@ -938,8 +942,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 // ColXpos
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("frame-col-xpos",pszColXpos))
+	if(!pSectionAP || !pSectionAP->getProperty("frame-col-xpos",pszColXpos))
 	{
 		m_iXColumn = 0;
 	}
@@ -950,8 +953,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	UT_DEBUGMSG(("ColXpos for frame is %s \n",pszColXpos));
 // colYpos
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("frame-col-ypos",pszColYpos))
+	if(!pSectionAP || !pSectionAP->getProperty("frame-col-ypos",pszColYpos))
 	{
 		m_iYColumn = 0;
 	}
@@ -964,8 +966,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 
 // PageXpos
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("frame-page-xpos",pszPageXpos))
+	if(!pSectionAP || !pSectionAP->getProperty("frame-page-xpos",pszPageXpos))
 	{
 		m_iXPage = 0;
 	}
@@ -976,8 +977,7 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	UT_DEBUGMSG(("PageXpos for frame is %s \n",pszPageXpos));
 // PageYpos
 
-	if((pView->getViewMode() == VIEW_NORMAL && !pG->queryProperties(GR_Graphics::DGP_PAPER)) ||
-	   !pSectionAP || !pSectionAP->getProperty("frame-page-ypos",pszPageYpos))
+	if(!pSectionAP || !pSectionAP->getProperty("frame-page-ypos",pszPageYpos))
 	{
 		m_iYPage = 0;
 	}
@@ -1110,6 +1110,20 @@ void fl_FrameLayout::_lookupProperties(const PP_AttrProp* pSectionAP)
 	{
 		m_iBoundingSpace = UT_convertToLogicalUnits(pszBoundingSpace);
 	}
+	//
+	// Preferred Page
+	//
+	if(!pSectionAP || !pSectionAP->getProperty("pref-page",pszPrefPage))
+	{
+		m_iPrefPage = -1;
+	}
+	else
+	{
+		if(pszPrefPage && *pszPrefPage != 0)
+			m_iPrefPage = atoi(pszPrefPage);
+		else
+			m_iPrefPage = -1;
+	}
 }
 
 void fl_FrameLayout::_lookupMarginProperties(const PP_AttrProp* pSectionAP)
@@ -1467,3 +1481,4 @@ static void s_border_properties (const gchar * border_color, const gchar * borde
 			line.m_thickness = static_cast<UT_sint32>(thickness / UT_PAPER_UNITS_PER_INCH);
 		}
 }
+
