@@ -34,35 +34,26 @@
 #include "xap_Frame.h"
 #include "ev_Mouse.h"
 #include "ut_locale.h"
-
-#define FRAME_HANDLE_SIZE 6
+#include "fp_FrameContainer.h"
 
 #define MIN_DRAG_PIXELS 8
 
 FV_VisualInlineImage::FV_VisualInlineImage (FV_View * pView)
-	: m_pView (pView), 
+	: FV_Base(pView), 
 	  m_iInlineDragMode(FV_InlineDrag_NOT_ACTIVE),
 	  m_pDragImage(NULL),
 	  m_iLastX(0),
 	  m_iLastY(0),
-	  m_recCurFrame(0,0,0,0),
 	  m_iInitialOffX(0),
 	  m_iInitialOffY(0),
-	  m_iFirstEverX(0),
-	  m_iFirstEverY(0),
 	  m_bTextCut(false),
 	  m_pDocUnderCursor(NULL),
 	  m_bCursorDrawn(false),
 	  m_recCursor(0,0,0,0),
 	  m_pAutoScrollTimer(NULL),
-	  m_xLastMouse(1),
-	  m_yLastMouse(1),
 	  m_bDoingCopy(false),
-	  m_iDraggingWhat( FV_Inline_DragNothing ),
-	  m_iGlobCount(0),
 	  m_pImageAP(NULL),
 	  m_screenCache(NULL),
-	  m_bFirstDragDone(false),
 	  m_bIsEmbedded(false)
 {
 	UT_ASSERT_HARMLESS(pView);
@@ -85,17 +76,16 @@ bool FV_VisualInlineImage::isActive(void) const
 	return (FV_InlineDrag_NOT_ACTIVE != m_iInlineDragMode);
 }
 
-GR_Graphics * FV_VisualInlineImage::getGraphics(void) const
-{
-	return m_pView->getGraphics();
-}
-
 void FV_VisualInlineImage::setMode(FV_InlineDragMode iEditMode)
 {
 	m_iInlineDragMode = iEditMode;
 }
 
-void FV_VisualInlineImage::_autoScroll(UT_Worker * pWorker)
+static UT_sint32 iExtra = 0;
+static bool bScrollRunning = false;
+static UT_Worker * s_pScroll = NULL;
+
+void FV_VisualInlineImage::_actuallyScroll(UT_Worker * pWorker)
 {
 	UT_return_if_fail(pWorker);
 
@@ -133,11 +123,11 @@ void FV_VisualInlineImage::_autoScroll(UT_Worker * pWorker)
 	{
 		if(bScrollUp)
 		{
-			pView->cmdScroll(AV_SCROLLCMD_LINEUP, static_cast<UT_uint32>( -y));
+			pView->cmdScroll(AV_SCROLLCMD_LINEUP, static_cast<UT_uint32>( -y)+iExtra);
 		}
 		else if(bScrollDown)
 		{
-			pView->cmdScroll(AV_SCROLLCMD_LINEDOWN, static_cast<UT_uint32>(y - pView->getWindowHeight()));
+			pView->cmdScroll(AV_SCROLLCMD_LINEDOWN, static_cast<UT_uint32>(y - pView->getWindowHeight())+iExtra);
 		}
 		if(bScrollLeft)
 		{
@@ -153,498 +143,266 @@ void FV_VisualInlineImage::_autoScroll(UT_Worker * pWorker)
 		pView->_setPoint(posAtXY);
 		pVis->drawCursor(posAtXY);
 #endif
+		iExtra = 0;
 		return;
 	}
 	else
 	{
-		pVis->m_pAutoScrollTimer->stop();
+		if(pVis->m_pAutoScrollTimer)
+			pVis->m_pAutoScrollTimer->stop();
 		DELETEP(pVis->m_pAutoScrollTimer);
 	}
+	s_pScroll->stop();
+	delete s_pScroll;
+	s_pScroll = NULL;
+	bScrollRunning = false;
+	iExtra = 0;
+}
+
+void FV_VisualInlineImage::_autoScroll(UT_Worker * pWorker)
+{
+	UT_return_if_fail(pWorker);
+
+	// this is a static callback method and does not have a 'this' pointer.
+
+	FV_VisualInlineImage * pVis = static_cast<FV_VisualInlineImage *>(pWorker->getInstanceData());
+	UT_return_if_fail(pVis);
+	if(bScrollRunning)
+	{
+	    UT_DEBUGMSG(("Dropping InlineImage autoscroll !!!!!!! \n"));
+	    FV_View * pView = pVis->m_pView;
+	    if(iExtra < pVis->getGraphics()->tlu(600))
+	      iExtra += pVis->getGraphics()->tlu(20);
+	    return;
+	}
+
+	int inMode = UT_WorkerFactory::IDLE | UT_WorkerFactory::TIMER;
+	UT_WorkerFactory::ConstructMode outMode = UT_WorkerFactory::NONE;
+	s_pScroll = UT_WorkerFactory::static_constructor (_actuallyScroll,pVis, inMode, outMode);
+
+	// If the worker is working on a timer instead of in the idle
+	// time, set the frequency of the checks.
+	if ( UT_WorkerFactory::TIMER == outMode )
+	{
+		// this is really a timer, so it's safe to static_cast it
+		static_cast<UT_Timer*>(s_pScroll)->set(100);
+	}
+	bScrollRunning = true;
+	s_pScroll->start();
+	iExtra = 0;
 
 }
 
-void FV_VisualInlineImage::mouseDrag(UT_sint32 x, UT_sint32 y)
+void FV_VisualInlineImage::_mouseDrag(UT_sint32 x, UT_sint32 y)
 {
 	GR_Graphics * pG = getGraphics();
-
-  if(m_iDraggingWhat == FV_Inline_DragWholeImage)
-  {
-        if(m_iInlineDragMode  == FV_InlineDrag_NOT_ACTIVE)
-        {
-	  m_iFirstEverX = x;
-	  m_iFirstEverY = y;
-	  m_iInlineDragMode  = FV_InlineDrag_WAIT_FOR_MOUSE_DRAG;
-	  xxx_UT_DEBUGMSG(("Initial call for drag -1\n"));
-	  return;
-	}
-	if((m_iFirstEverX == 0) && (m_iFirstEverY == 0))
+	if(getDragWhat() == FV_DragWhole)
 	{
-	  m_iFirstEverX = x;
-	  m_iFirstEverY = y;
-	  m_iInlineDragMode = FV_InlineDrag_WAIT_FOR_MOUSE_DRAG;
-	  xxx_UT_DEBUGMSG(("Initial call for drag -2 \n"));
-	}
-	if(m_iInlineDragMode == FV_InlineDrag_WAIT_FOR_MOUSE_DRAG)
-	{
-          float diff = sqrt((static_cast<float>(x) - static_cast<float>(m_iFirstEverX))*(static_cast<float>(x) - static_cast<float>(m_iFirstEverX)) +
-                              (static_cast<float>(y) - static_cast<float>(m_iFirstEverY))*(static_cast<float>(y) - static_cast<float>(m_iFirstEverY)));
-          if(diff < static_cast<float>(pG->tlu(MIN_DRAG_PIXELS)))
-          {
-	    xxx_UT_DEBUGMSG(("Not yet dragged enough.%f \n", diff));
-            //
-            // Have to drag 4 pixels before initiating the drag
-            //
-            return;
-          }
-	  else
-	  {
-	    m_iInlineDragMode = FV_InlineDrag_START_DRAGGING;	    
-	  }
-        }
-	m_bFirstDragDone = true;
-	if((m_iInlineDragMode != FV_InlineDrag_DRAGGING) && (m_iInlineDragMode != FV_InlineDrag_WAIT_FOR_MOUSE_DRAG) && !m_bDoingCopy)
-	{
-//
-// Haven't started the drag yet so create our image and cut the text.
-//
-	  _beginGlob();
-	  mouseCut(m_iFirstEverX,m_iFirstEverY);
-	  m_bTextCut = true;
-	}
-	clearCursor();
-	m_iInlineDragMode = FV_InlineDrag_DRAGGING;
-	xxx_UT_DEBUGMSG(("x = %d y = %d width \n",x,y));
-	bool bScrollDown = false;
-	bool bScrollUp = false;
-	bool bScrollLeft = false;
-	bool bScrollRight = false;
-	m_xLastMouse = x;
-	m_yLastMouse = y;
-	if(y<=0)
-	{
-		bScrollUp = true;
-	}
-	else if( y >= m_pView->getWindowHeight())
-	{
-		bScrollDown = true;
-	}
-	if(x <= 0)
-	{
-		bScrollLeft = true;
-	}
-	else if(x >= m_pView->getWindowWidth())
-	{
-		bScrollRight = true;
-	}
-	if(bScrollDown || bScrollUp || bScrollLeft || bScrollRight)
-	{
-		if(m_pAutoScrollTimer != NULL)
+    	if(m_iInlineDragMode  == FV_InlineDrag_NOT_ACTIVE)
 		{
+			m_iFirstEverX = x;
+			m_iFirstEverY = y;
+			m_iInlineDragMode  = FV_InlineDrag_WAIT_FOR_MOUSE_DRAG;
+			xxx_UT_DEBUGMSG(("Initial call for drag -1\n"));
 			return;
 		}
-		m_pAutoScrollTimer = UT_Timer::static_constructor(_autoScroll, this);
-		m_pAutoScrollTimer->set(AUTO_SCROLL_MSECS);
-		m_pAutoScrollTimer->start();
-		return;
-	}
-	UT_sint32 dx = 0;
-	UT_sint32 dy = 0;
-	UT_Rect expX(0,m_recCurFrame.top,0,m_recCurFrame.height);
-	UT_Rect expY(m_recCurFrame.left,0,m_recCurFrame.width,0);
-	UT_sint32 iext = pG->tlu(3);
-	dx = x - m_iLastX;
-	dy = y - m_iLastY;
-	m_recCurFrame.left += dx;
-	m_recCurFrame.top += dy;
-	if(dx < 0)
-	{
-		expX.left = m_recCurFrame.left+m_recCurFrame.width -iext;
-		expX.width = -dx + 2*iext;
-		if(dy > 0)
+		if((m_iFirstEverX == 0) && (m_iFirstEverY == 0))
 		{
-			expX.top -=  iext;
-			expX.height += dy + 2*iext;
+			m_iFirstEverX = x;
+			m_iFirstEverY = y;
+			m_iInlineDragMode = FV_InlineDrag_WAIT_FOR_MOUSE_DRAG;
+			xxx_UT_DEBUGMSG(("Initial call for drag -2 \n"));
+		}
+		if(m_iInlineDragMode == FV_InlineDrag_WAIT_FOR_MOUSE_DRAG)
+		{
+			float diff = sqrt(
+				(static_cast<float>(x) - static_cast<float>(m_iFirstEverX))*(static_cast<float>(x) - static_cast<float>(m_iFirstEverX)) +
+				(static_cast<float>(y) - static_cast<float>(m_iFirstEverY))*(static_cast<float>(y) - static_cast<float>(m_iFirstEverY))
+							  );
+			if(diff < static_cast<float>(pG->tlu(MIN_DRAG_PIXELS)))
+			{
+				xxx_UT_DEBUGMSG(("Not yet dragged enough.%f \n", diff));
+				//
+				// Have to drag 4 pixels before initiating the drag
+				//
+				return;
+			}
+			else
+			{
+				m_iInlineDragMode = FV_InlineDrag_START_DRAGGING;	    
+			}
+		}
+		m_bFirstDragDone = true;
+		if((m_iInlineDragMode != FV_InlineDrag_DRAGGING) && (m_iInlineDragMode != FV_InlineDrag_WAIT_FOR_MOUSE_DRAG) && !m_bDoingCopy)
+		{
+			//
+			// Haven't started the drag yet so create our image and cut the text.
+			//
+			_beginGlob();
+			mouseCut(m_iFirstEverX,m_iFirstEverY);
+			m_bTextCut = true;
+		}
+		clearCursor();
+		m_iInlineDragMode = FV_InlineDrag_DRAGGING;
+		xxx_UT_DEBUGMSG(("x = %d y = %d width \n",x,y));
+		bool bScrollDown = false;
+		bool bScrollUp = false;
+		bool bScrollLeft = false;
+		bool bScrollRight = false;
+		m_xLastMouse = x;
+		m_yLastMouse = y;
+		if(y<=0)
+		{
+			bScrollUp = true;
+		}
+		else if( y >= m_pView->getWindowHeight())
+		{
+			bScrollDown = true;
+		}
+		if(x <= 0)
+		{
+			bScrollLeft = true;
+		}
+		else if(x >= m_pView->getWindowWidth())
+		{
+			bScrollRight = true;
+		}
+		if(bScrollDown || bScrollUp || bScrollLeft || bScrollRight)
+		{
+			if(m_pAutoScrollTimer != NULL)
+			{
+				return;
+			}
+			m_pAutoScrollTimer = UT_Timer::static_constructor(_autoScroll, this);
+			m_pAutoScrollTimer->set(AUTO_SCROLL_MSECS);
+			m_pAutoScrollTimer->start();
+			return;
+		}
+		UT_sint32 dx = 0;
+		UT_sint32 dy = 0;
+		UT_Rect expX(0,m_recCurFrame.top,0,m_recCurFrame.height);
+		UT_Rect expY(m_recCurFrame.left,0,m_recCurFrame.width,0);
+		UT_sint32 iext = pG->tlu(3);
+		dx = x - m_iLastX;
+		dy = y - m_iLastY;
+		m_recCurFrame.left += dx;
+		m_recCurFrame.top += dy;
+		if(dx < 0)
+		{
+			expX.left = m_recCurFrame.left+m_recCurFrame.width -iext;
+			expX.width = -dx + 2*iext;
+			if(dy > 0)
+			{
+				expX.top -=  iext;
+				expX.height += dy + 2*iext;
+			}
+			else
+			{
+				expX.top -=  iext;
+				expX.height += (-dy + 2*iext);
+			}
 		}
 		else
 		{
-			expX.top -=  iext;
-			expX.height += (-dy + 2*iext);
+			expX.left = m_recCurFrame.left - dx - iext;
+			expX.width = dx + 2*iext;
+			if(dy > 0)
+			{
+				expX.top -=  iext;
+				expX.height += dy + 2*iext;
+			}
+			else
+			{
+				expX.top -= iext;
+				expX.height += (-dy + 2*iext);
+			}
 		}
-	}
-	else
-	{
-		expX.left = m_recCurFrame.left - dx - iext;
-		expX.width = dx + 2*iext;
-		if(dy > 0)
+		expY.left -= iext;
+		expY.width += 2*iext;
+		if(dy < 0)
 		{
-			expX.top -=  iext;
-			expX.height += dy + 2*iext;
+			expY.top = m_recCurFrame.top + m_recCurFrame.height -iext;
+			expY.height = -dy + 2*iext;
 		}
 		else
 		{
-			expX.top -= iext;
-			expX.height += (-dy + 2*iext);
+			expY.top = m_recCurFrame.top - dy - iext;
+			expY.height = dy + 2*iext;
 		}
-	}
-	expY.left -= iext;
-	expY.width += 2*iext;
-	if(dy < 0)
-	{
-		expY.top = m_recCurFrame.top + m_recCurFrame.height -iext;
-		expY.height = -dy + 2*iext;
-	}
-	else
-	{
-		expY.top = m_recCurFrame.top - dy - iext;
-		expY.height = dy + 2*iext;
-	}
 
-	if(expX.width > 0)
-	{
-		pG->setClipRect(&expX);
-		m_pView->updateScreen(false);
+		if(expX.width > 0)
+		{
+			pG->setClipRect(&expX);
+			m_pView->updateScreen(false);
+		}
+		if(expY.height > 0)
+		{
+			pG->setClipRect(&expY);
+			m_pView->updateScreen(false);
+		}
+		pG->setClipRect(NULL);
+		bool b = drawImage();
+		if(!b)
+		{
+			cleanUP();
+			return;
+		}
+		m_iLastX = x;
+		m_iLastY = y;
+		pG->setClipRect(NULL);
+		PT_DocPosition posAtXY = getPosFromXY(x,y);
+		m_pView->_setPoint(posAtXY);
+		//m_pView->_fixInsertionPointCoords();
+		drawCursor(posAtXY);
 	}
-	if(expY.height > 0)
+	else
 	{
-		pG->setClipRect(&expY);
-		m_pView->updateScreen(false);
-	}
-	pG->setClipRect(NULL);
-	bool b = drawImage();
-	if(!b)
-	{
-	  cleanUP();
-	  return;
-	}
-	m_iLastX = x;
-	m_iLastY = y;
-	pG->setClipRect(NULL);
-	PT_DocPosition posAtXY = getPosFromXY(x,y);
-	m_pView->_setPoint(posAtXY);
-//	m_pView->_fixInsertionPointCoords();
-	drawCursor(posAtXY);
-  }
-  else
-  {
         m_iInlineDragMode = FV_InlineDrag_RESIZE;
-	if(!m_bFirstDragDone)
-	{
-		m_iFirstEverX = x;
-		m_iFirstEverY = y;
-	}
-	m_bFirstDragDone = true;
-	UT_sint32 diffx = 0;
-	UT_sint32 diffy = 0;
-	UT_sint32 dx = 0;
-	UT_sint32 dy = 0;
-	UT_Rect prevRect = m_recCurFrame;
-	UT_Rect expX(0,m_recCurFrame.top,0,m_recCurFrame.height);
-	UT_Rect expY(m_recCurFrame.left,0,m_recCurFrame.width,0);
-	UT_sint32 iext = pG->tlu(3);
-	m_xLastMouse = x;
-	m_yLastMouse = y;
-	switch (m_iDraggingWhat)
-	{
-	case FV_Inline_DragTopLeftCorner:
-		diffx = m_recCurFrame.left - x;
-		diffy = m_recCurFrame.top - y;
-		m_recCurFrame.left -= diffx;
-		m_recCurFrame.top -= diffy;
-		dx = -diffx;
-		dy = -diffy;
-		m_recCurFrame.width += diffx;
-		m_recCurFrame.height += diffy;
-		if(diffx < 0)
+		UT_Rect prevRect = m_recCurFrame;
+		UT_sint32 dx = 0;
+		UT_sint32 dy = 0;
+		UT_Rect expX(0,m_recCurFrame.top,0,m_recCurFrame.height);
+		UT_Rect expY(m_recCurFrame.left,0,m_recCurFrame.width,0);
+		FV_Base::_doMouseDrag( x, y, dx, dy, expX, expY );
+		_checkDimensions();
+		
+		if(expX.width > 0)
 		{
-			expX.left = m_recCurFrame.left + diffx -iext;
-			expX.width = -diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  diffy + iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
+			pG->setClipRect(&expX);
+			m_pView->updateScreen(false);
 		}
-		if(diffy < 0)
+		if(expY.height > 0)
 		{
-			expY.top = m_recCurFrame.top + diffy - iext;
-			expY.height = -diffy + 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NE);
-			m_iDraggingWhat =  FV_Inline_DragTopRightCorner;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SW);
-			m_iDraggingWhat =  FV_Inline_DragBotLeftCorner;
-		}
-		break;
-	case FV_Inline_DragTopRightCorner:
-		diffx = m_recCurFrame.left + m_recCurFrame.width - x;
-		diffy = m_recCurFrame.top - y;
-		m_recCurFrame.top -= diffy;
-		dy = -diffy;
-		m_recCurFrame.width -= diffx;
-		m_recCurFrame.height += diffy;
-		if(diffx > 0)
-		{
-			expX.left = m_recCurFrame.left + m_recCurFrame.width;
-			expX.width = diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy < 0)
-		{
-			expY.top = m_recCurFrame.top + diffy - iext;
-			expY.height = -diffy + iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NW);
-			m_iDraggingWhat =  FV_Inline_DragTopLeftCorner;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SE);
-			m_iDraggingWhat =  FV_Inline_DragBotRightCorner;
-		}
-		break;
-	case FV_Inline_DragBotLeftCorner:
-		diffx = m_recCurFrame.left - x;
-		diffy = m_recCurFrame.top + m_recCurFrame.height - y;
-		m_recCurFrame.left -= diffx;
-		dx = -diffx;
-		m_recCurFrame.width += diffx;
-		m_recCurFrame.height -= diffy;
-		if(diffx < 0)
-		{
-			expX.left = m_recCurFrame.left + diffx -iext;
-			expX.width = -diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  diffy + iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy > 0)
-		{
-			expY.top = m_recCurFrame.top + m_recCurFrame.height - iext;
-			expY.height = diffy + 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SE);
-			m_iDraggingWhat =  FV_Inline_DragBotRightCorner;
-
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NW);
-			m_iDraggingWhat =  FV_Inline_DragTopLeftCorner;
-		}
-		break;
-	case FV_Inline_DragBotRightCorner:
-		diffx = m_recCurFrame.left + m_recCurFrame.width - x;
-		diffy = m_recCurFrame.top + m_recCurFrame.height - y;
-		m_recCurFrame.width -= diffx;
-		m_recCurFrame.height -= diffy;
-		if(diffx > 0)
-		{
-			expX.left = m_recCurFrame.left + m_recCurFrame.width;
-			expX.width = diffx + iext;
-			if(diffy > 0)
-			{
-				expX.top -=  iext;
-				expX.height += diffy + 2*iext;
-			}
-			else
-			{
-				expX.top -=  iext;
-				expX.height += (-diffy + 2*iext);
-			}
-		}
-		if(diffy > 0)
-		{
-			expY.top = m_recCurFrame.top + m_recCurFrame.height;
-			expY.height = diffy + iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_SW);
-			m_iDraggingWhat =  FV_Inline_DragBotLeftCorner;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_NE);
-			m_iDraggingWhat =  FV_Inline_DragTopRightCorner;
-		}
-		break;
-	case FV_Inline_DragLeftEdge:
-		diffx = m_recCurFrame.left - x;
-		m_recCurFrame.left -= diffx;
-		dx = -diffx;
-		m_recCurFrame.width += diffx;
-		if(diffx < 0)
-		{
-			expX.left = m_recCurFrame.left + diffx - iext;
-			expX.width = -diffx + iext;
-			expX.top -=  iext;
-			expX.height += 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_W);
-			m_iDraggingWhat =  FV_Inline_DragRightEdge;
-		}
-		break;
-	case FV_Inline_DragRightEdge:
-		diffx = m_recCurFrame.left + m_recCurFrame.width - x;
-		m_recCurFrame.width -= diffx;
-		if(diffx > 0)
-		{
-			expX.left = m_recCurFrame.left + m_recCurFrame.width;
-			expX.width = diffx + iext;
-			expX.top -=  iext;
-			expX.height += 2*iext;
-		}
-		if(m_recCurFrame.width < 0)
-		{
-			m_recCurFrame.left = x;
-			m_recCurFrame.width = -m_recCurFrame.width;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_E);
-			m_iDraggingWhat =  FV_Inline_DragLeftEdge;
-		}
-		break;
-	case FV_Inline_DragTopEdge:
-		diffy = m_recCurFrame.top - y;
-		m_recCurFrame.top -= diffy;
-		dy = -diffy;
-		m_recCurFrame.height += diffy;
-		if(diffy < 0)
-		{
-			expY.top = m_recCurFrame.top + diffy - iext;
-			expY.height = -diffy + iext;
-			expY.left -= iext;
-			expY.width += 2*iext;
-		}
-		if(m_recCurFrame.height < 0)
-		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_S);
-			m_iDraggingWhat =  FV_Inline_DragBotEdge;
-		}
-		break;
-	case FV_Inline_DragBotEdge:
-		diffy = m_recCurFrame.top + m_recCurFrame.height - y;
-		m_recCurFrame.height -= diffy;
-		if(diffy > 0)
-		{
-			expY.top = m_recCurFrame.top + m_recCurFrame.height;
-			expY.height = diffy + iext;
-			expY.left -= iext;
-			expY.width += 2*iext;
+			pG->setClipRect(&expY);
 			xxx_UT_DEBUGMSG(("expY.top %d expY.height %d \n",expY.top,expY.height));
+			m_pView->updateScreen(false);
 		}
-		if(m_recCurFrame.height < 0)
+		pG->setClipRect(NULL);
+		GR_Painter painter(pG);
+		//
+		// Clear the previous line.
+		//
+		if(m_screenCache != NULL)
 		{
-			m_recCurFrame.top = y;
-			m_recCurFrame.height = -m_recCurFrame.height;
-			pG->setCursor(GR_Graphics::GR_CURSOR_IMAGESIZE_N);
-			m_iDraggingWhat =  FV_Inline_DragTopEdge;
+			prevRect.left -= pG->tlu(1);
+			prevRect.top -= pG->tlu(1);
+			painter.drawImage(m_screenCache,prevRect.left,prevRect.top);
+			DELETEP(m_screenCache);
 		}
-		break;
-	default:
-	  break;
-	}
-	// don't left widths and heights be too big
+		//
+		// Save the current screen
+		//
+		UT_Rect rCache = m_recCurFrame;
+		rCache.left -= pG->tlu(1);
+		rCache.top -= pG->tlu(1);
+		rCache.width += pG->tlu(2);
+		rCache.height += pG->tlu(2);
+		m_screenCache = painter.genImageFromRectangle(rCache);
 
-	double dWidth = static_cast<double>(m_recCurFrame.width)/static_cast<double>(UT_LAYOUT_RESOLUTION);
-	double dHeight = static_cast<double>(m_recCurFrame.height)/static_cast<double>(UT_LAYOUT_RESOLUTION);
-	if(m_pView->getPageSize().Width(DIM_IN) < dWidth)
-	{
-	  dWidth = m_pView->getPageSize().Width(DIM_IN)*0.99;
-	  m_recCurFrame.width = static_cast<UT_sint32>(dWidth*UT_LAYOUT_RESOLUTION);
+		// Draw new image box
+		UT_Rect box(m_recCurFrame.left, m_recCurFrame.top - pG->tlu(1), m_recCurFrame.width - pG->tlu(1), m_recCurFrame.height - pG->tlu(1));
+		m_pView->drawSelectionBox(box, false);
 	}
-	if(m_pView->getPageSize().Height(DIM_IN) < dHeight)
-	{
-	  dHeight = m_pView->getPageSize().Height(DIM_IN)*0.99;
-	  m_recCurFrame.height = static_cast<UT_sint32>(dHeight*UT_LAYOUT_RESOLUTION);
-	}
-	if(expX.width > 0)
-	{
-	        pG->setClipRect(&expX);
-		m_pView->updateScreen(false);
-	}
-	if(expY.height > 0)
-	{
-	        pG->setClipRect(&expY);
-		xxx_UT_DEBUGMSG(("expY.top %d expY.height %d \n",expY.top,expY.height));
-		m_pView->updateScreen(false);
-	}
-	pG->setClipRect(NULL);
-	GR_Painter painter(pG);
-	//
-	// Clear the previous line.
-	//
-	if(m_screenCache != NULL)
-	{
-	  prevRect.left -= pG->tlu(1);
-	  prevRect.top -= pG->tlu(1);
-	  painter.drawImage(m_screenCache,prevRect.left,prevRect.top);
-	  DELETEP(m_screenCache);
-	}
-	//
-	// Save the current screen
-	//
-	UT_Rect rCache = m_recCurFrame;
-	rCache.left -= pG->tlu(1);
-	rCache.top -= pG->tlu(1);
-	rCache.width += pG->tlu(2);
-	rCache.height += pG->tlu(2);
-	m_screenCache = painter.genImageFromRectangle(rCache);
-
-	// Draw new image box
-	UT_Rect box(m_recCurFrame.left, m_recCurFrame.top - pG->tlu(1), m_recCurFrame.width - pG->tlu(1), m_recCurFrame.height - pG->tlu(1));
-	m_pView->drawSelectionBox(box, false);
-  }
 }
 
 void FV_VisualInlineImage::clearCursor(void)
@@ -769,7 +527,7 @@ void FV_VisualInlineImage::getImageFromSelection(UT_sint32 x, UT_sint32 y,PP_Att
 	}
 	UT_sint32 xoff = 0, yoff = 0;
 	pRun->getLine()->getScreenOffsets(pRun, xoff, yoff);
-	UT_DEBUGMSG(("@@@@ ===== Screen offsets %d, %d; line ascent %d, run ascent %d\n",
+	xxx_UT_DEBUGMSG(("@@@@ ===== Screen offsets %d, %d; line ascent %d, run ascent %d\n",
 		     xoff, yoff, pRun->getLine()->getAscent(), pRun->getAscent()));
 	// Sevior's infamous + 1....
 	yoff += pRun->getLine()->getAscent() - pRun->getAscent() + getGraphics()->tlu(1);	// Set the image size in the image selection rect
@@ -791,6 +549,12 @@ void FV_VisualInlineImage::getImageFromSelection(UT_sint32 x, UT_sint32 y,PP_Att
 	}
 }
 
+const char * FV_VisualInlineImage::getPNGImage(const UT_ByteBuf ** pBuf)
+{
+        m_pView->getDocument()->getDataItemDataByName(m_sDataId.utf8_str(),pBuf,NULL,NULL);
+	return m_sDataId.utf8_str();
+}
+
 void FV_VisualInlineImage::mouseCut(UT_sint32 x, UT_sint32 y)
 {
 	getImageFromSelection(x,y);
@@ -831,6 +595,10 @@ void FV_VisualInlineImage::mouseCut(UT_sint32 x, UT_sint32 y)
 		}
 		if(pRun && ((pRun->getType() == FPRUN_IMAGE) || ((pRun->getType() == FPRUN_EMBED))))
 		{
+		        if(pRun->getType() == FPRUN_IMAGE)
+			{
+			    m_sDataId = static_cast<fp_ImageRun *>(pRun)->getDataId();
+			}
 		        posLow = pBlock->getPosition() + pRun->getBlockOffset();
 			// we've found an image: do not move the view, just select the image and exit
 			m_pView->cmdSelect(posLow,posLow+1);
@@ -843,37 +611,6 @@ void FV_VisualInlineImage::mouseCut(UT_sint32 x, UT_sint32 y)
 	drawImage();
 }
 
-
-UT_sint32 FV_VisualInlineImage::getGlobCount()
-{
-	return m_iGlobCount;
-}
-
-void  FV_VisualInlineImage::_beginGlob(void)
-{
-	getDoc()->beginUserAtomicGlob();
-	m_iGlobCount++;
-	xxx_UT_DEBUGMSG(("Begin Glob count %d \n",m_iGlobCount));
-}
-
-
-void  FV_VisualInlineImage::_endGlob(void)
-{
-	getDoc()->endUserAtomicGlob();
-	m_iGlobCount--;
-	xxx_UT_DEBUGMSG(("End Glob count %d \n",m_iGlobCount));
-}
-
-PD_Document *    FV_VisualInlineImage::getDoc(void) const
-{
-	return m_pView->getDocument();
-}
-
-FL_DocLayout *   FV_VisualInlineImage::getLayout(void) const
-{
-        return m_pView->getLayout();
-}
-
 void FV_VisualInlineImage::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 {
 	if(!isActive())
@@ -917,14 +654,14 @@ void FV_VisualInlineImage::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 	{
 	        m_iInlineDragMode = FV_InlineDrag_WAIT_FOR_MOUSE_DRAG;
 		setDragType(x,y,false); // was true
-		if(FV_Inline_DragNothing == m_iDraggingWhat)
+		if(FV_DragNothing ==  getDragWhat())
 		{
 		  cleanUP();
 		  m_pView->warpInsPtToXY(x,y,true);
 		}
 		else
 		{
-			if( m_iDraggingWhat != FV_Inline_DragWholeImage)
+			if( getDragWhat() != FV_DragWhole)
 			{
 				m_iInlineDragMode = FV_InlineDrag_RESIZE;
 			}
@@ -945,10 +682,17 @@ void FV_VisualInlineImage::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 	drawImage();
 }
 
+
+void FV_VisualInlineImage::abortDrag(void)
+{
+  cleanUP();
+  m_pView->cmdUndo(1);
+}
+
 void FV_VisualInlineImage::cleanUP(void)
 {
   m_iInlineDragMode = FV_InlineDrag_NOT_ACTIVE;
-  m_iDraggingWhat = FV_Inline_DragNothing;
+  setDragWhat( FV_DragNothing );
   DELETEP(m_pDragImage);
   DELETEP(m_pDocUnderCursor);
   DELETEP(m_screenCache);
@@ -1015,7 +759,7 @@ void FV_VisualInlineImage::setDragType(UT_sint32 x,UT_sint32 y, bool bDrawImage)
 	{
 	  return;
 	}
-	UT_sint32 ires = getGraphics()->tlu(FRAME_HANDLE_SIZE); // 6 pixels wide hit area
+	UT_sint32 ires = getGraphics()->tlu(FRAME_HANDLE_SIZE);
 	UT_sint32 iLeft = m_recCurFrame.left;
 	UT_sint32 iRight = m_recCurFrame.left + m_recCurFrame.width;
 	UT_sint32 iTop = m_recCurFrame.top;
@@ -1026,81 +770,79 @@ void FV_VisualInlineImage::setDragType(UT_sint32 x,UT_sint32 y, bool bDrawImage)
 	bool bRight = (iRight - ires < x) && (x < iRight + ires);
 	bool bTop = (iTop - ires < y) && (y < iTop + ires);
 	bool bBot = (iBot - ires < y) && (y < iBot + ires);
-
 //
 // Not resizeable embedded object
 //
 	if (m_bIsEmbedded && !m_bEmbedCanResize)
 	{
-		m_iDraggingWhat = FV_Inline_DragWholeImage;
+		setDragWhat( FV_DragWhole );
 	}
 //
 // top left
 //
 	else if((iLeft < x) && (x < iLeft + ires) && (iTop < y) && (y < iTop + ires))
 	{
-		m_iDraggingWhat = FV_Inline_DragTopLeftCorner;
+		setDragWhat( FV_DragTopLeftCorner );
 	}
 //
 // top Right
 //
 	else if((iRight - ires < x) && (x < iRight) && (iTop < y) && (y < iTop + ires))
 	{
-		m_iDraggingWhat = FV_Inline_DragTopRightCorner;
+		setDragWhat( FV_DragTopRightCorner );
 	}
 //
 // bot left
 //
 	else if((iLeft < x) && (x < iLeft + ires) && (iBot > y) && (y > iBot - ires))
 	{
-		m_iDraggingWhat = FV_Inline_DragBotLeftCorner;
+		setDragWhat( FV_DragBotLeftCorner );
 	}
-
 //
 // Bot Right
 //
 	else if((iRight - ires < x) && (x < iRight) && (iBot > y) && (y > iBot - ires))
 	{
-		m_iDraggingWhat = FV_Inline_DragBotRightCorner;
+		setDragWhat( FV_DragBotRightCorner );
 	}
 //
 // top Edge
 //
 	else if( bX && bTop)
 	{
-		m_iDraggingWhat = FV_Inline_DragTopEdge;
+		setDragWhat( FV_DragTopEdge );
 	}
 //
 // left Edge
 //
 	else if(bLeft && bY)
 	{
-		m_iDraggingWhat = FV_Inline_DragLeftEdge;
+		setDragWhat( FV_DragLeftEdge );
 	}
 //
 // right Edge
 //
 	else if(bRight && bY)
 	{
-		m_iDraggingWhat = FV_Inline_DragRightEdge;
+		setDragWhat( FV_DragRightEdge );
 	}
 //
 // bot Edge
 //
 	else if(bBot && bX)
 	{
-		m_iDraggingWhat = FV_Inline_DragBotEdge;
+		setDragWhat( FV_DragBotEdge );
 	}
 	else
 	{
 		if( bX && bY)
 		{
-			m_iDraggingWhat = FV_Inline_DragWholeImage;
-			xxx_UT_DEBUGMSG(("Dragging Whole Image \n"));
+			setDragWhat( FV_DragWhole );
+			xxx_UT_DEBUGMSG(("!!!!!Dragging Whole Image \n"));
 		}
 		else
 		{
-			m_iDraggingWhat = FV_Inline_DragNothing;
+			setDragWhat( FV_DragNothing );
 			return;
 		}
 	}
@@ -1111,7 +853,7 @@ void FV_VisualInlineImage::setDragType(UT_sint32 x,UT_sint32 y, bool bDrawImage)
 	m_iLastX = x;
 	m_iLastY = y;
 	xxx_UT_DEBUGMSG(("Initial width %d height %d \n",m_recCurFrame.width,m_recCurFrame.height));
-	xxx_UT_DEBUGMSG((" Dragging What %d \n",m_iDraggingWhat));
+	xxx_UT_DEBUGMSG((" Dragging What %d \n",getDragWhat()));
 	m_pView->setCursorToContext();
 }
 
@@ -1162,7 +904,7 @@ void FV_VisualInlineImage::mouseCopy(UT_sint32 x, UT_sint32 y)
 	  }
 	}
 	m_iInlineDragMode= FV_InlineDrag_START_DRAGGING;
-	m_iDraggingWhat = FV_Inline_DragWholeImage;
+	setDragWhat( FV_DragWhole );
 	getImageFromSelection(x,y);
 	m_pView->m_prevMouseContext = EV_EMC_IMAGESIZE;
 	m_pView->setCursorToContext();
@@ -1226,39 +968,16 @@ void FV_VisualInlineImage::mouseRelease(UT_sint32 x, UT_sint32 y)
 	clearCursor();
 	if(((m_iInlineDragMode != FV_InlineDrag_DRAGGING) && (m_iInlineDragMode != FV_InlineDrag_RESIZE) ) || !m_bFirstDragDone)
 	{
-	     PT_DocPosition pos = m_pView->getDocPositionFromXY(x,y);
-	     fl_BlockLayout * pBlock = m_pView->getBlockAtPosition(pos);
-	     if(pBlock)
-	     {
-		   UT_sint32 x1,x2,y1,y2,iHeight;
-		   bool bEOL = false;
-		   bool bDir = false;
-		
-		   fp_Run * pRun = NULL;
-		   
-		   pRun = pBlock->findPointCoords(pos,bEOL,x1,y1,x2,y2,iHeight,bDir);
-		   while(pRun && ((pRun->getType() != FPRUN_IMAGE) && (pRun->getType() != FPRUN_EMBED)))
-		   {
-			pRun = pRun->getNextRun();
-		   }
-		   if(pRun && ((pRun->getType() == FPRUN_IMAGE) || ((pRun->getType() == FPRUN_EMBED))))
-		   {
-			m_bFirstDragDone = false;
-			return;
-		   }
-		   else
-		   {
 //
-// we didn't actually drag anything. Just release the selection.
+// we didn't actually drag anything far enough. Just release the selection.
 //
-		        cleanUP();
-			m_pView->warpInsPtToXY(x, y,true);
-			return;
-		   }
-	     }
+	     cleanUP();
+	     m_pView->warpInsPtToXY(x, y,true);
+	     return;
+
 	}
 	m_bFirstDragDone = false;
-	if(FV_Inline_DragWholeImage == m_iDraggingWhat)
+	if(FV_DragWhole ==  getDragWhat())
 	{
 	  PT_DocPosition posAtXY = getPosFromXY(x,y);
 	  m_pView->setPoint(posAtXY);
@@ -1317,26 +1036,31 @@ void FV_VisualInlineImage::mouseRelease(UT_sint32 x, UT_sint32 y)
 	    UT_String_setProperty(sProps,sProp,sVal);
 	  }
 	  bFound = m_pImageAP->getProperty("height",szHeight);
-	  if(!bFound)
+	  if(bFound)
 	  {
 	    sProp = "height";
 	    sVal = szHeight;
 	    UT_String_setProperty(sProps,sProp,sVal);
 	  }
-
-	  bFound = m_pImageAP->getProperty("title",szTitle);
-	  bFound = m_pImageAP->getProperty("alt",szDescription);
-	  sProps += "; height:";
-	  sProps += szHeight;
+	  bFound = m_pImageAP->getAttribute("title",szTitle);
+	  if(!bFound)
+	  {
+	    szTitle = "";
+	  }
+	  bFound = m_pImageAP->getAttribute("alt",szDescription);
+	  if(!bFound)
+	  {
+	    szDescription = "";
+	  }
 	  const gchar*	attributes[] = {
 	    "dataid", NULL,
-	    "title",NULL,
-	    "alt",NULL,
 	    PT_PROPS_ATTRIBUTE_NAME, NULL,
+	    PT_IMAGE_TITLE,NULL,
+	    PT_IMAGE_DESCRIPTION,NULL,
 	    NULL,NULL};
 	  attributes[1] = szDataID;
-	  attributes[3] = szTitle;
-	  attributes[5] = szDescription;
+	  attributes[5] = szTitle;
+	  attributes[7] = szDescription;
 	  if(m_bIsEmbedded)
 	  {
 	      sProp="embed-type";
@@ -1345,14 +1069,14 @@ void FV_VisualInlineImage::mouseRelease(UT_sint32 x, UT_sint32 y)
 	  }
 	  if(sProps.size() > 0)
 	  {
-	    attributes[7] = sProps.c_str();
+	    attributes[3] = sProps.c_str();
 	  }
 	  else
 	  {
-	    attributes[6] = NULL;
+	    attributes[2] = NULL;
 	  }
 	  m_pView->_saveAndNotifyPieceTableChange();
-	  xxx_UT_DEBUGMSG(("Doing Insert Image at %d \n",m_pView->getPoint()));
+	  UT_DEBUGMSG(("Doing Insert Image at %d \n",m_pView->getPoint()));
 	  if(!m_bIsEmbedded)
 	  {
 	    getDoc()->insertObject(m_pView->getPoint(), PTO_Image, attributes, NULL);
@@ -1384,8 +1108,8 @@ void FV_VisualInlineImage::mouseRelease(UT_sint32 x, UT_sint32 y)
 	  UT_Rect newImgBounds = m_recCurFrame;
 	  const fp_PageSize & page = m_pView->getPageSize ();		
 	  double max_width = 0., max_height = 0.;
-	  max_width  = page.Width (DIM_PX);
-	  max_height = page.Height (DIM_PX);
+	  max_width  = page.Width (DIM_IN)*UT_LAYOUT_RESOLUTION;
+	  max_height = page.Height (DIM_IN)*UT_LAYOUT_RESOLUTION;
 		
 	  // some range checking stuff
 	  newImgBounds.width = abs(newImgBounds.width);
@@ -1427,8 +1151,8 @@ void FV_VisualInlineImage::mouseRelease(UT_sint32 x, UT_sint32 y)
 
 	    {
 	      UT_LocaleTransactor t(LC_NUMERIC, "C");
-	      UT_UTF8String_sprintf(sWidth, "%fin", UT_convertDimToInches(newImgBounds.width, DIM_PX));
-	      UT_UTF8String_sprintf(sHeight, "%fin", UT_convertDimToInches(newImgBounds.height, DIM_PX));
+	      UT_UTF8String_sprintf(sWidth, "%fin", static_cast<double>(newImgBounds.width)/UT_LAYOUT_RESOLUTION);
+	      UT_UTF8String_sprintf(sHeight, "%fin", static_cast<double>(newImgBounds.height)/UT_LAYOUT_RESOLUTION);
 	    }
 	    properties[1] = sWidth.utf8_str();
 	    properties[3] = sHeight.utf8_str();
@@ -1449,3 +1173,8 @@ bool FV_VisualInlineImage::drawImage(void)
 	painter.drawImage(m_pDragImage,m_recCurFrame.left,m_recCurFrame.top);
 	return true;
 }
+
+UT_sint32 FV_VisualInlineImage::getImageSelBoxSize() const
+{
+	return FRAME_HANDLE_SIZE;
+}
