@@ -198,7 +198,7 @@ static PP_Property _props[] =
 	{ "toc-has-label2",       "1",                false, NULL, PP_LEVEL_BLOCK},
 	{ "toc-has-label3",       "1",                false, NULL, PP_LEVEL_BLOCK},
 	{ "toc-has-label4",       "1",                false, NULL, PP_LEVEL_BLOCK},
-	{ "toc-heading",       "Table of Contents",   false, NULL, PP_LEVEL_BLOCK},
+	{ "toc-heading",       "Contents",   false, NULL, PP_LEVEL_BLOCK},
 	{ "toc-heading-style",  "Contents Header",    false, NULL, PP_LEVEL_BLOCK},
     { "toc-id",                "0",               false, NULL, PP_LEVEL_SECT},
 	{ "toc-indent1",           "0.5in",           false, NULL, PP_LEVEL_BLOCK},
@@ -249,9 +249,6 @@ static PP_Property _props[] =
 	{ "wrap-mode",             "above-text",      false, NULL, PP_LEVEL_FRAME} 
 };
 
-#ifdef __MRC__
-extern "C"
-#endif
 static int s_compare (const void * a, const void * b)
 {
   const PP_Property * prop;
@@ -269,7 +266,7 @@ const PP_Property * PP_lookupProperty(const gchar * name)
 {
 	PP_Property * prop = NULL;
 
-	prop = static_cast<PP_Property *>(bsearch (name, _props, NrElements(_props), sizeof (_props[0]), s_compare));
+	prop = static_cast<PP_Property *>(bsearch (name, _props, G_N_ELEMENTS(_props), sizeof (_props[0]), s_compare));
 
 	return prop;
 }
@@ -278,7 +275,7 @@ const PP_Property * PP_lookupProperty(const gchar * name)
 void PP_resetInitialBiDiValues(const gchar * pszValue)
 {
 	int i;
-	int count = NrElements(_props);
+	int count = G_N_ELEMENTS(_props);
 
 	for (i=0; i<count; i++)
 	{
@@ -305,7 +302,7 @@ void PP_resetInitialBiDiValues(const gchar * pszValue)
 void PP_setDefaultFontFamily(const char* pszFamily)
 {
 	static UT_String family(pszFamily);
-	PP_Property* prop = static_cast<PP_Property*>(bsearch ("font-family", _props, NrElements(_props), sizeof(_props[0]), s_compare));
+	PP_Property* prop = static_cast<PP_Property*>(bsearch ("font-family", _props, G_N_ELEMENTS(_props), sizeof(_props[0]), s_compare));
 	prop->m_pszInitial = const_cast<gchar*>(reinterpret_cast<const gchar*>(family.c_str()));
 }
 
